/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package scalapi.itext

import java.io.ByteArrayOutputStream
import java.util.regex.Pattern
import com.lowagie.text.{Paragraph, Document}
import com.lowagie.text.pdf.PdfWriter
import scalapi.jdk.control.TakeWhile

trait PdfTemplate {

  // Constants

  private val placeholdersPattern = Pattern.compile("\\$\\{(.*?)\\}")

  // Members

  private val documentBytes = new ByteArrayOutputStream
  private val document = new Document

  // Constructor

  PdfWriter.getInstance(document, documentBytes)
  document.open()

  // Abstract members

  def parameters: Map[String, String]

  // Export

  def toBytes: Array[Byte] = {
    document.close()
    documentBytes.toByteArray
  }

  // DSL


  def title(title: String) {
    document addTitle prepareText(title)
  }

  def subject(subject: String) {
    document addSubject prepareText(subject)
  }

  def paragraph(text: String) {
    document add new Paragraph(prepareText(text))
  }

  // Helpers

  private def prepareText(text: String): String = {
    val cleanText = text.stripMargin
    val matcher = placeholdersPattern.matcher(cleanText)
    TakeWhile(matcher.find) {
      (matcher.group, matcher.group(1))
    }().foldLeft(cleanText) {
      (text, placeholder) => text.replaceAllLiterally(placeholder._1, parameters(placeholder._2))
    }
  }

}